@@ -21,14 +21,14 @@ ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */
 
-/*! @example sample7.cpp
+/* @example sample7.cpp
  This example demonstrates using of memory save mode of bitset operations.
 
 For more information please visit:  http://bmagic.sourceforge.net
 
 */
 
-/*! @file
+/* @file
     @ingroup mset
 */
 
@@ -37,6 +37,14 @@ For more information please visit:  http://bmagic.sourceforge.net
 #include <stdio.h>
 #include <time.h>
 
+// Example disabled as deprecated...
+
+int main()
+{
+    return 1;
+}
+
+/*
 
 // BM library version 3.1.3 and later can keep internal bit flags in pointers.
 // This is efficient but not completely portable hack. 
@@ -178,7 +186,7 @@ int main(void)
     OrSets();
 
     time_t end_time = time(0);
-    unsigned elapsed = unsigned(end_time - start_time);
+    time_t elapsed = end_time - start_time;
     cout << "elapsed=" << elapsed << endl;
     unsigned ops;
     if (elapsed)
@@ -193,5 +201,5 @@ int main(void)
 
     return 0;
 }
-
+*/
   
