@@ -32,6 +32,12 @@ OTHER DEALINGS IN THE SOFTWARE.
 //#define BMSSE42OPT
 //#define BM64OPT
 
+#ifdef _MSC_VER
+#pragma warning( push )
+#pragma warning( disable : 4996)
+#endif
+
+
 #include "bm.h"
 #include "bmalgo.h"
 #include "bmserial.h"
@@ -1232,6 +1238,9 @@ int main(void)
 }
 
 
+#ifdef _MSC_VER
+#pragma warning( pop )
+#endif
 
 
 
